/*
 * game.c
 *
 *  Created on: Apr 28, 2018
 *      Author: wn
 */






#include <stdlib.h>
#include <stdbool.h>

#include "PontCoopScheduler.h"

#include "leds.h"
#include "game.h"
#include "myRand.h"



tCellState field[FIELD_SIZE];




static void setDirectCell(uint8_t cell, tCellState state) {
	field[cell] = state;
}

static void setCell(uint8_t row, uint8_t col, tCellState state) {
	field[(row * ROW_SIZE) + col] = state;
}

static tCellState getDirectCell(uint8_t cell) {
	return field[cell];
}

static tCellState getCell(uint8_t row, uint8_t col) {
	return field[(row * ROW_SIZE) + col];
}

static void setField(tCellState s) {
	for (uint8_t i = 0; i < FIELD_SIZE; i++) {
		field[i] = s;
	}
}

static void play(tCellState state) {
	while (true) {
		uint8_t r = myRand();
		if (getDirectCell(r) == csEMPTY) {
			setDirectCell(r, state);
			break;
		}
	}
}


static void playCircle() {
	play(csCIRCLE);
}

static void playCross() {
	play(csCROSS);
}


typedef enum { arNOWINNER, arCROSS, arCIRCLE, arGAMEOVER } tArbiterResult;

static tArbiterResult mapCellStateToArbiterResult(tCellState c) {
	tArbiterResult a;
	if (c == csEMPTY) {
		a = arNOWINNER;
	} else if (c == csCROSS) {
		a = arCROSS;
	} else if (c == csCIRCLE) {
		a = arCIRCLE;
	}
	return a;
}

static tArbiterResult arbiter() {
	tCellState baseState;

	// checks rows
	for (uint8_t row = 0; row < ROW_SIZE; row++) {
		baseState = getCell(row, 0);
		if (baseState == csEMPTY) {
			continue;
		}
		if ((getCell(row, 1) == baseState) && getCell(row, 2) == baseState) {
			return mapCellStateToArbiterResult(baseState);
		}
	}

	// checks cols
	for (uint8_t col = 0; col < COL_SIZE; col++) {
		baseState = getCell(0, col);
		if (baseState == csEMPTY) {
			continue;
		}
		if ((getCell(1, col) == baseState) && getCell(2, col) == baseState) {
			return mapCellStateToArbiterResult(baseState);
		}
	}

	// checks diagonal 1
	baseState = getCell(0, 0);
	if ((baseState != csEMPTY) &&
		(getCell(1, 1) == baseState) &&
		(getCell(2, 2) == baseState)) {
		return mapCellStateToArbiterResult(baseState);
	}

	// checks diagonal 2
	baseState = getCell(2, 0);
	if ((baseState != csEMPTY) &&
		(getCell(1, 1) == baseState) &&
		(getCell(0, 2) == baseState)) {
		return mapCellStateToArbiterResult(baseState);
	}

	// check for free fields
	for (uint8_t i = 0; i < FIELD_SIZE; i++) {
		if (getDirectCell(i) == csEMPTY) {
			return mapCellStateToArbiterResult(csEMPTY);
		}
	}

	return arGAMEOVER;
}


void gameMove() {
	static enum { sNEW, sCROSS, sCIRCLE, sARBITER1, sARBITER2 } state = sNEW;
	tArbiterResult winner;
	switch (state) {
	case sNEW:
		setField(csEMPTY);
		state = sCROSS;
		break;
	case sCROSS:
		playCross();
		state = sARBITER1;
		break;
	case sARBITER1:
		winner = arbiter();
		if (winner == arNOWINNER) {
			state = sCIRCLE;
		} else if (winner == arGAMEOVER) {
			setField(csEMPTY);
			state = sNEW;
		} else {
			setField(winner);
			state = sNEW;
		}
		break;
	case sCIRCLE:
		playCircle();
		state = sARBITER2;
		break;
	case sARBITER2:
		winner = arbiter();
		if (winner == arNOWINNER) {
			state = sCROSS;
		} else if (winner == arGAMEOVER) {
			setField(csEMPTY);
			state = sNEW;
		} else {
			setField(winner);
			state = sNEW;
		}
		break;
	}
}


void gameInit() {
	setField(csEMPTY);
	schAdd(gameMove, NULL, 0, 100);
}

tCellState* gameGetField() {
	return field;
}
